@@ -1,6 +1,4 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
 # coding=utf8
-
 # Copyright 2014 IBM Corporation
 # Copyright 2015 Lenovo
 #
@@ -16,30 +14,44 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# This module provides access to SDR offered by a BMC
-# This data is common between 'sensors' and 'inventory' modules since SDR
-# is both used to enumerate sensors for sensor commands and FRU ids for FRU
-# commands
+"""This module provides access to SDR offered by a BMC
 
-# For now, we will not offer persistent SDR caching as we do in xCAT's IPMI
-# code.  Will see if it is adequate to advocate for high object reuse in a
-# persistent process for the moment.
+This data is common between 'sensors' and 'inventory' modules since SDR
+is both used to enumerate sensors for sensor commands and FRU ids for FRU
+commands
 
-# Focus is at least initially on the aspects that make the most sense for a
-# remote client to care about.  For example, smbus information is being
-# skipped for now
+For now, we will not offer persistent SDR caching as we do in xCAT's IPMI
+code.  Will see if it is adequate to advocate for high object reuse in a
+persistent process for the moment.
+
+Focus is at least initially on the aspects that make the most sense for a
+remote client to care about.  For example, smbus information is being
+skipped for now
+"""
 
 import math
+import os
+import random
+import string
+import struct
+import weakref
+
 import pyghmi.constants as const
 import pyghmi.exceptions as exc
 import pyghmi.ipmi.private.constants as ipmiconst
-import struct
-import weakref
+import six
+
+try:
+    import cPickle as pickle
+except ImportError:
+    import pickle
 
 TYPE_UNKNOWN = 0
 TYPE_SENSOR = 1
 TYPE_FRU = 2
 
+shared_sdrs = {}
+
 
 def ones_complement(value, bits):
     # utility function to help with the large amount of 2s
@@ -254,6 +266,7 @@ class SDREntry(object):
     def __init__(self, entrybytes, ipmicmd, reportunsupported=False):
         # ignore record id for now, we only care about the sensor number for
         # moment
+        self.readable = True
         self.reportunsupported = reportunsupported
         self.ipmicmd = ipmicmd
         if entrybytes[2] != 0x51:
@@ -267,6 +280,8 @@ class SDREntry(object):
             self.full_decode(entrybytes[5:])
         elif self.rectype == 2:  # full sdr
             self.compact_decode(entrybytes[5:])
+        elif self.rectype == 3:  # event only
+            self.eventonly_decode(entrybytes[5:])
         elif self.rectype == 8:  # entity association
             self.association_decode(entrybytes[5:])
         elif self.rectype == 0x11:  # FRU locator
@@ -315,6 +330,12 @@ class SDREntry(object):
         # TODO(jbjohnso): actually represent this data
         self.sdrtype = TYPE_UNKNOWN
 
+    def eventonly_decode(self, entry):
+        # table 43-3 event_only sensor record
+        self._common_decode(entry)
+        self.sensor_name = self.tlv_decode(entry[11], entry[12:])
+        self.readable = False
+
     def compact_decode(self, entry):
         # table 43-2 compact sensor record
         self._common_decode(entry)
@@ -325,18 +346,26 @@ class SDREntry(object):
         return trapval + offset
 
     def _common_decode(self, entry):
-        # compact and full are very similar
+        # event only, compact and full are very similar
         # this function handles the common aspects of compact and full
         # offsets from spec, minus 6
+        self.sensor_lun = entry[1] & 0x03
         self.sensor_number = entry[2]
         self.entity = ipmiconst.entity_ids.get(
             entry[3], 'Unknown entity {0}'.format(entry[3]))
-        self.sensor_type_number = entry[7]
+        if self.rectype == 3:
+            self.sensor_type_number = entry[5]
+            self.reading_type = entry[6]  # table 42-1
+        else:
+            self.sensor_type_number = entry[7]
+            self.reading_type = entry[8]  # table 42-1
         try:
-            self.sensor_type = ipmiconst.sensor_type_codes[entry[7]]
+            self.sensor_type = ipmiconst.sensor_type_codes[
+                self.sensor_type_number]
         except KeyError:
-            self.sensor_type = "UNKNOWN type " + str(entry[7])
-        self.reading_type = entry[8]  # table 42-1
+            self.sensor_type = "UNKNOWN type " + str(self.sensor_type_number)
+        if self.rectype == 3:
+            return
         # 0: unspecified
         # 1: generic threshold based
         # 0x6f: discrete sensor-specific from table 42-3, sensor offsets
@@ -365,6 +394,13 @@ class SDREntry(object):
         self.percent = ''
         if entry[15] & 1 == 1:
             self.percent = '% '
+        if self.sensor_type_number == 0xb:
+            if self.unit_mod == '':
+                if entry[16] == 6:
+                    self.sensor_type = 'Power'
+            elif self.unit_mod == ' ':
+                if entry[16] == 6 and entry[17] in (22, 23, 24):
+                    self.sensor_type = 'Energy'
         self.baseunit = unit_types[entry[16]]
         self.modunit = unit_types[entry[17]]
         self.unit_suffix = self.percent + self.baseunit + self.unit_mod + \
@@ -529,18 +565,15 @@ class SDREntry(object):
         elif linearization == 10:
             return math.sqrt(decoded)
         elif linearization == 11:
-            return decoded ** (1.0/3)
+            return decoded ** (1.0 / 3)
         else:
             raise NotImplementedError
 
     def decode_formula(self, entry):
-        self.m = \
-            twos_complement(entry[0] + ((entry[1] & 0b11000000) << 2), 10)
+        self.m = twos_complement(entry[0] + ((entry[1] & 0b11000000) << 2), 10)
         self.tolerance = entry[1] & 0b111111
-        self.b = \
-            twos_complement(entry[2] + ((entry[3] & 0b11000000) << 2), 10)
-        self.accuracy = (entry[3] & 0b111111) + \
-            (entry[4] & 0b11110000) << 2
+        self.b = twos_complement(entry[2] + ((entry[3] & 0b11000000) << 2), 10)
+        self.accuracy = (entry[3] & 0b111111) + (entry[4] & 0b11110000) << 2
         self.accuracyexp = (entry[4] & 0b1100) >> 2
         self.direction = entry[4] & 0b11
         # 0 = n/a, 1 = input, 2 = output
@@ -556,7 +589,8 @@ class SDREntry(object):
             return ""
         if ipmitype == 0:  # Unicode per 43.15 in ipmi 2.0 spec
             # the spec is not specific about encoding, assuming utf8
-            return unicode(struct.pack("%dB" % len(data), *data), "utf_8")
+            return six.text_type(struct.pack("%dB" % len(data), *data),
+                                 "utf_8")
         elif ipmitype == 1:  # BCD '+'
             tmpl = "%02X" * len(data)
             tstr = tmpl % tuple(data)
@@ -573,9 +607,14 @@ class SDREntry(object):
                 tstr += chr(((data[2] & 0b11) << 4) +
                             (data[1] >> 4) + 0x20)
                 tstr += chr((data[2] >> 2) + 0x20)
+            if not isinstance(tstr, str):
+                tstr = tstr.decode('utf-8')
             return tstr
         elif ipmitype == 3:  # ACSII+LATIN1
-            return struct.pack("%dB" % len(data), *data)
+            ret = struct.pack("%dB" % len(data), *data)
+            if not isinstance(ret, str):
+                ret = ret.decode('utf-8')
+            return ret
 
 
 class SDR(object):
@@ -587,10 +626,11 @@ class SDR(object):
 
     :param ipmicmd: A Command class object
     """
-    def __init__(self, ipmicmd):
+    def __init__(self, ipmicmd, cachedir=None):
         self.ipmicmd = weakref.proxy(ipmicmd)
         self.sensors = {}
         self.fru = {}
+        self.cachedir = cachedir
         self.read_info()
 
     def read_info(self):
@@ -604,9 +644,9 @@ class SDR(object):
         self.fw_major = rsp['data'][2] & 0b1111111
         self.fw_minor = "%02X" % rsp['data'][3]  # BCD encoding, oddly enough
         self.ipmiversion = rsp['data'][4]  # 51h = 1.5, 02h = 2.0
-        self.mfg_id = rsp['data'][8] << 16 + rsp['data'][7] << 8 + \
+        self.mfg_id = (rsp['data'][8] << 16) + (rsp['data'][7] << 8) + \
             rsp['data'][6]
-        self.prod_id = rsp['data'][10] << 8 + rsp['data'][9]
+        self.prod_id = (rsp['data'][10] << 8) + rsp['data'][9]
         if len(rsp['data']) > 11:
             self.aux_fw = self.decode_aux(rsp['data'][11:15])
         if rsp['data'][1] & 0b10000000 and rsp['data'][5] & 0b10 == 0:
@@ -647,12 +687,46 @@ class SDR(object):
         # NOTE(jbjohnso): not looking to support the various options in op
         # support, ignore those for now, reservation if some BMCs can't read
         # full SDR in one slurp
+        modtime = struct.unpack('!Q', bytes(repinfo['data'][5:13]))[0]
         recid = 0
         rsvid = 0  # partial 'get sdr' will require this
         offset = 0
         size = 0xff
         chunksize = 128
+        try:
+            csdrs = shared_sdrs[
+                (self.fw_major, self.fw_minor, self.mfg_id, self.prod_id,
+                 self.device_id, modtime)]
+            self.sensors = csdrs['sensors']
+            self.fru = csdrs['fru']
+            return
+        except KeyError:
+            pass
+        cachefilename = None
         self.broken_sensor_ids = {}
+        if self.cachedir:
+            cachefilename = 'sdrcache.{0}.{1}.{2}.{3}.{4}.{5}'.format(
+                self.mfg_id, self.prod_id, self.device_id, self.fw_major,
+                self.fw_minor, modtime)
+            cachefilename = os.path.join(self.cachedir, cachefilename)
+        if cachefilename and os.path.isfile(cachefilename):
+            with open(cachefilename, 'rb') as cfile:
+                csdrs = pickle.load(cfile)
+                for sdrdata in csdrs:
+                    self.add_sdr(sdrdata)
+                for sid in self.broken_sensor_ids:
+                    try:
+                        del self.sensors[sid]
+                    except KeyError:
+                        pass
+                shared_sdrs[
+                    (self.fw_major, self.fw_minor, self.mfg_id, self.prod_id,
+                     self.device_id, modtime)] = {
+                    'sensors': self.sensors,
+                    'fru': self.fru,
+                }
+                return
+        sdrraw = [] if cachefilename else None
         while recid != 0xffff:  # per 33.12 Get SDR command, 0xffff marks end
             newrecid = 0
             currlen = 0
@@ -697,6 +771,8 @@ class SDR(object):
                 if (offset + size) > currlen:
                     size = currlen - offset
             self.add_sdr(sdrdata)
+            if sdrraw is not None:
+                sdrraw.append(sdrdata)
             offset = 0
             if size != 0xff:
                 size = 5
@@ -708,14 +784,27 @@ class SDR(object):
                 del self.sensors[sid]
             except KeyError:
                 pass
+        shared_sdrs[(self.fw_major, self.fw_minor, self.mfg_id, self.prod_id,
+                     self.device_id, modtime)] = {
+            'sensors': self.sensors,
+            'fru': self.fru,
+        }
+        if cachefilename:
+            suffix = ''.join(
+                random.choice(string.ascii_lowercase) for _ in range(12))
+            with open(cachefilename + '.' + suffix, 'wb') as cfile:
+                pickle.dump(sdrraw, cfile)
+            os.rename(cachefilename + '.' + suffix, cachefilename)
 
     def get_sensor_numbers(self):
-        return self.sensors.iterkeys()
+        for number in self.sensors:
+            if self.sensors[number].readable:
+                yield number
 
     def add_sdr(self, sdrbytes):
         newent = SDREntry(sdrbytes, self.ipmicmd)
         if newent.sdrtype == TYPE_SENSOR:
-            id = newent.sensor_number
+            id = '{0}.{1}'.format(newent.sensor_number, newent.sensor_lun)
             if id in self.sensors:
                 self.broken_sensor_ids[id] = True
                 return
@@ -731,20 +820,3 @@ class SDR(object):
         # This is where manufacturers can add their own
         # decode information
         return "".join(hex(x) for x in auxdata)
-
-if __name__ == "__main__":  # test code
-    import os
-    import pyghmi.ipmi.command as ipmicmd
-    import sys
-    password = os.environ['IPMIPASSWORD']
-    bmc = sys.argv[1]
-    user = sys.argv[2]
-    ipmicmd = ipmicmd.Command(bmc=bmc, userid=user, password=password)
-    sdr = SDR(ipmicmd)
-    for number in sdr.get_sensor_numbers():
-        rsp = ipmicmd.raw_command(command=0x2d, netfn=4, data=(number,))
-        if 'error' in rsp:
-            continue
-        reading = sdr.sensors[number].decode_sensor_reading(rsp['data'])
-        if reading is not None:
-            print(repr(reading))
