@@ -1,4 +1,4 @@
-# Copyright 2015-2017 Lenovo
+# Copyright 2015-2019 Lenovo
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,26 +16,28 @@
 # sake of typical internal management devices.  Compatibility back to python
 # 2.6 as is found in commonly used enterprise linux distributions.
 
+import base64
+import copy
+import gzip
+import io
 import json
-import pyghmi.exceptions as pygexc
 import socket
 import ssl
 import threading
 
+import pyghmi.exceptions as pygexc
+import six
+
 try:
     import Cookie
     import httplib
-    import StringIO
 except ImportError:
     import http.client as httplib
     import http.cookies as Cookie
-    import io as StringIO
-
-__author__ = 'jjohnson2'
 
 
 # Used as the separator for form data
-BND = 'TbqbLUSn0QFjx9gxiQLtgBK4Zu6ehLqtLs4JOBS50EgxXJ2yoRMhTrmRXxO1lkoAQdZx16'
+BND = b'TbqbLUSn0QFjx9gxiQLtgBK4Zu6ehLqtLs4JOBS50EgxXJ2yoRMhTrmRXxO1lkoAQdZx16'
 
 # We will frequently be dealing with the same data across many instances,
 # consolidate forms to single memory location to get benefits..
@@ -52,11 +54,18 @@ class FileUploader(threading.Thread):
         self.data = data
         self.otherfields = otherfields
         self.formname = formname
+        self.rsp = ''
+        self.rspstatus = 500
         super(FileUploader, self).__init__()
 
     def run(self):
-        self.rsp = self.wc.upload(self.url, self.filename, self.data,
-                                  self.formname, otherfields=self.otherfields)
+        try:
+            self.rsp = self.wc.upload(
+                self.url, self.filename, self.data, self.formname,
+                otherfields=self.otherfields)
+        except Exception:
+            self.rspstatus = self.wc.rspstatus
+            raise
 
 
 class FileDownloader(threading.Thread):
@@ -65,10 +74,15 @@ class FileDownloader(threading.Thread):
         self.wc = webclient
         self.url = url
         self.savefile = savefile
+        self.exc = None
         super(FileDownloader, self).__init__()
 
     def run(self):
-        self.wc.download(self.url, self.savefile)
+        try:
+            self.wc.download(self.url, self.savefile)
+        except Exception as e:
+            self.exc = e
+
 
 def get_upload_form(filename, data, formname, otherfields):
     if not formname:
@@ -80,14 +94,19 @@ def get_upload_form(filename, data, formname, otherfields):
             data = data.read()
         except AttributeError:
             pass
-        form = '--' + BND + '\r\nContent-Disposition: form-data; ' \
-                            'name="{0}"; filename="{1}"\r\n'.format(formname,
-                                                                    filename)
-        form += 'Content-Type: application/octet-stream\r\n\r\n' + data
+        form = (b'--' +
+                BND +
+                '\r\nContent-Disposition: form-data; '
+                'name="{0}"; filename="{1}"\r\n'.format(
+                    formname, filename).encode('utf-8'))
+        form += b'Content-Type: application/octet-stream\r\n\r\n' + data
         for ofield in otherfields:
-            form += '\r\n--' + BND + '\r\nContent-Disposition: form-data; ' \
-                'name="{0}"\r\n\r\n{1}'.format(ofield, otherfields[ofield])
-        form += '\r\n--' + BND + '--\r\n'
+            form += (b'\r\n--' +
+                     BND +
+                     '\r\nContent-Disposition: form-data; '
+                     'name="{0}"\r\n\r\n{1}'.format(
+                         ofield, otherfields[ofield]).encode('utf-8'))
+        form += b'\r\n--' + BND + b'--\r\n'
         uploadforms[filename] = form
         return form
 
@@ -100,33 +119,55 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
                  **kwargs):
         if 'timeout' not in kwargs:
             kwargs['timeout'] = 60
+        self.mytimeout = kwargs['timeout']
+        self._currdl = None
         self.lastjsonerror = None
         self.broken = False
         self.thehost = host
         self.theport = port
-        httplib.HTTPConnection.__init__(self, host, port, strict, **kwargs)
+        try:
+            httplib.HTTPConnection.__init__(self, host, port, strict=strict,
+                                            **kwargs)
+        except TypeError:
+            httplib.HTTPConnection.__init__(self, host, port, **kwargs)
         self.cert_reqs = ssl.CERT_NONE  # verification will be done ssh style..
         if clone:
             self._certverify = clone._certverify
             self.cookies = clone.cookies
-            self.stdheaders = clone.stdheaders
+            self.stdheaders = copy.deepcopy(clone.stdheaders)
         else:
             self._certverify = verifycallback
             self.cookies = {}
             self.stdheaders = {}
+        if '[' not in host and '%' in host:
+            self.stdheaders['Host'] = '[' + host[:host.find('%')] + ']'
 
     def dupe(self):
-        return SecureHTTPConnection(self.thehost, self.theport, clone=self)
+        return SecureHTTPConnection(self.thehost, self.theport, clone=self,
+                                    timeout=self.mytimeout)
 
     def set_header(self, key, value):
         self.stdheaders[key] = value
 
+    def set_basic_credentials(self, username, password):
+        if isinstance(username, bytes) and not isinstance(username, str):
+            username = username.decode('utf-8')
+        if isinstance(password, bytes) and not isinstance(password, str):
+            password = password.decode('utf-8')
+        authinfo = ':'.join((username, password))
+        if not isinstance(authinfo, bytes):
+            authinfo = authinfo.encode('utf-8')
+        authinfo = base64.b64encode(authinfo)
+        if not isinstance(authinfo, str):
+            authinfo = authinfo.decode('utf-8')
+        self.stdheaders['Authorization'] = 'Basic {0}'.format(authinfo)
+
     def connect(self):
         addrinfo = socket.getaddrinfo(self.host, self.port)[0]
         # workaround problems of too large mtu, moderately frequent occurance
         # in this space
         plainsock = socket.socket(addrinfo[0])
-        plainsock.settimeout(60)
+        plainsock.settimeout(self.mytimeout)
         try:
             plainsock.setsockopt(socket.IPPROTO_TCP, socket.TCP_MAXSEG, 1456)
         except socket.error:
@@ -142,9 +183,13 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
     def getresponse(self):
         try:
             rsp = super(SecureHTTPConnection, self).getresponse()
-            for hdr in rsp.msg.headers:
-                if hdr.startswith('Set-Cookie:'):
-                    c = Cookie.BaseCookie(hdr[11:])
+            try:
+                hdrs = [x.split(':', 1) for x in rsp.msg.headers]
+            except AttributeError:
+                hdrs = rsp.msg.items()
+            for hdr in hdrs:
+                if hdr[0] == 'Set-Cookie':
+                    c = Cookie.BaseCookie(hdr[1])
                     for k in c:
                         self.cookies[k] = c[k].value
         except httplib.BadStatusLine:
@@ -154,33 +199,62 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
 
     def grab_json_response(self, url, data=None, referer=None, headers=None):
         self.lastjsonerror = None
+        body, status = self.grab_json_response_with_status(
+            url, data, referer, headers)
+        if status == 200:
+            return body
+        self.lastjsonerror = body
+        return {}
+
+    def grab_json_response_with_status(self, url, data=None, referer=None,
+                                       headers=None, method=None):
         webclient = self.dupe()
         if isinstance(data, dict):
             data = json.dumps(data)
         if data:
-            webclient.request('POST', url, data, referer=referer,
+            if not method:
+                method = 'POST'
+            webclient.request(method, url, data, referer=referer,
                               headers=headers)
         else:
-            webclient.request('GET', url, referer=referer, headers=headers)
-        rsp = webclient.getresponse()
-        if rsp.status == 200:
-            return json.loads(rsp.read())
-        self.lastjsonerror = rsp.read()
-        return {}
+            if not method:
+                method = 'GET'
+            webclient.request(method, url, referer=referer, headers=headers)
+        try:
+            rsp = webclient.getresponse()
+        except httplib.BadStatusLine:
+            return 'Target Unavailable', 500
+        except ssl.SSLError as e:
+            if 'timed out' in str(e):
+                return 'Target Unavailable', 500
+            raise
+        body = rsp.read()
+        if rsp.getheader('Content-Encoding', None) == 'gzip':
+            body = gzip.GzipFile(fileobj=io.BytesIO(body)).read()
+        if rsp.status >= 200 and rsp.status < 300:
+            if body and not isinstance(body, str):
+                body = body.decode('utf8')
+            return json.loads(body) if body else {}, rsp.status
+        return body, rsp.status
 
     def download(self, url, file):
         """Download a file to filename or file object
 
         """
-        if isinstance(file, str) or isinstance(file, unicode):
+        if isinstance(file, six.string_types):
             file = open(file, 'wb')
         webclient = self.dupe()
-        webclient.request('GET', url)
+        dlheaders = self.stdheaders.copy()
+        if 'Accept-Encoding' in dlheaders:
+            del dlheaders['Accept-Encoding']
+        webclient.request('GET', url, headers=dlheaders)
         rsp = webclient.getresponse()
         self._currdl = rsp
         self._dlfile = file
-        for chunk in iter(lambda: rsp.read(16384), ''):
+        chunk = rsp.read(16384)
+        while chunk:
             file.write(chunk)
+            chunk = rsp.read(16384)
         self._currdl = None
         file.close()
 
@@ -202,11 +276,10 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
         """
         if data is None:
             data = open(filename, 'rb')
-        self._upbuffer = StringIO.StringIO(get_upload_form(filename, data,
-                                                           formname,
-                                                           otherfields))
+        self._upbuffer = io.BytesIO(get_upload_form(
+            filename, data, formname, otherfields))
         ulheaders = self.stdheaders.copy()
-        ulheaders['Content-Type'] = 'multipart/form-data; boundary=' + BND
+        ulheaders['Content-Type'] = b'multipart/form-data; boundary=' + BND
         ulheaders['Content-Length'] = len(uploadforms[filename])
         self.ulsize = len(uploadforms[filename])
         webclient = self.dupe()
@@ -219,10 +292,14 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
             del uploadforms[filename]
         except KeyError:  # something could have already deleted it
             pass
+        self.rspstatus = rsp.status
         if rsp.status != 200:
             raise Exception('Unexpected response in file upload: ' +
                             rsp.read())
-        return rsp.read()
+        body = rsp.read()
+        if rsp.getheader('Content-Encoding', None) == 'gzip':
+            body = gzip.GzipFile(fileobj=io.BytesIO(body)).read()
+        return body
 
     def get_upload_progress(self):
         return float(self._upbuffer.tell()) / float(self.ulsize)
@@ -230,10 +307,14 @@ class SecureHTTPConnection(httplib.HTTPConnection, object):
     def request(self, method, url, body=None, headers=None, referer=None):
         if headers is None:
             headers = self.stdheaders.copy()
+        else:
+            headers = headers.copy()
         if method == 'GET' and 'Content-Type' in headers:
             del headers['Content-Type']
         if method == 'POST' and body and 'Content-Type' not in headers:
             headers['Content-Type'] = 'application/x-www-form-urlencoded'
+        if body and 'Content-Length' not in headers:
+            headers['Content-Length'] = len(body)
         if self.cookies:
             cookies = []
             for ckey in self.cookies:
